#!/usr/bin/perl

$version= "2.2";

# MAIN

use Getopt::Long;
require IO::File;

my $arm=10; # number of harmonics to check against to
my $quartz=10000000; # default quartz frequency 10MHz
my $mix=1; # not used
my $if=0; # extra IF value to check; 0=skip
my $sampleR=48000; #defalt SDRadio sampling rate, 48kHz

GetOptions (	"help",
		"arm=i" => \$arm,
		"quartz=i" => \$quartz,
		"file=s" => \$infile,
		"sample=i" => \$sampleR,
		"if=i" => \$if,
		"all" => \$all,
		"dcrx" => \$dcrx,
		"ifa" => \$ifa,
		"ifb" => \$ifb,
		"ifc" => \$ifc,
		"sdr" => \$sdr,
		"drma" => \$drma,
		"drmd" => \$drmd,
		"drmc" => \$drmc,
		"drmdc" => \$drmdc,
		"icsa" => \$icsa,
		"icsb" => \$icsb,
		"mcw" => \$mcw
		);


if ( $opt_help ) {
	printbanner();
	print "\nUsage: xtalfind.pl [-quartz <n>|-file <filename>] [-arm <n>] [-if <n>] [-sample <n>]\n";
	print "                   (-all|(-dcrx -ifa -ifb -ifc -sdr -drma -drmd -drmc -drmdc -icsa -icsb))\n";
	print "Valid options:\n";
	print "-help      	 		Get (this) help\n";
	print "-quartz=<number>		XTAL frequency in Hz (default 10000000 Hz)\n";
	print "-file=<filename>		Read XTAL list from file\n";
	print "-arm=<number>			Check matches up to -arm harmonic (default 10)\n";
	print "-if=<number>			Also check against a custom IF value, in Hz\n";
	print "-sample=<number>		Soundcard sample rate for SDR (default 48000 Hz)\n";
	print "-all   				Compute all available layouts -OR-\n";
	print "-dcrx				Compute for DC RX only\n";
	print "-ifa -ifb -ifc			Compute for 455kHz, 10.7MHz, 12kHz IF\n";
	print "-sdr				Compute for SDR\n";
	print "-drma -drmc -drmd		Compute for 455-12, 10.7-12, 9.01-12 DRM converter\n";
	print "-icsa -icsb			Compute for ICS512 or ICS672\n";
	print "-mcw				Compute for MCW optical system\n";
	print "\n\n";
	print "Specify AT LEAST one of the compute modes, -all or one of others\n";
	print "\n\n";
	exit;
}

# Check if the DRMfreq.txt list file is present
# if not, ignore DRM tests
$ff = new IO::File;
if ($ff->open("< DRMfreq.txt")) {
        # We've go the file, load it up
        $/="\n";
        $q=0;
        while (<$ff>) {
                chomp;
                $qrg[$q++] = $_;
        }
        $ff->close();
	$haveDRM=1;
} else {
	$haveDRM=0;
}


# Do we have an input file?
if ($infile) {
	$/="\n";
	$fh = new IO::File;
	
	# Can I open the file?
	if ($fh->open("< $infile")) {
		printbanner();
		while (<$fh>) {
			$quartz=$_;
			submain();
		}
	# OPS! File not found! Bailing out...
	} else {
		printbanner();
		printf("ERROR: could not open file $infile ... Aborting.\n\n");
		
	}
# A file name was not provided, going into single XTAL mode
} else {
	printbanner();
	submain();
}


# *** TODO: insert here parameters validity check


sub submain() {

my $i;

printf("\n------------------------------------------------------\n");
printf("      ******* Checking %d Hz quartz *******\n",$quartz);
printf("------------------------------------------------------\n");

if ($all || $dcrx) {
	printf("\n>>> Results for a TX or a direct-conversion RX <<<\n");
	$i=comp($quartz,0,$arm);
}

if ($all || $ifa) {
	printf("\n>>> Results for an RX with 455kHz IF <<<\n");
	$i=comp($quartz,455000,$arm);
}

if ($all || $ifb) {
	printf("\n>>> Results for an RX with 10.7MHz IF <<<\n");
	$i=comp($quartz,10700000,$arm);
}

if ($all || $ifc) {
	printf("\n>>> Results for an RX with 12kHz IF <<<\n");
	$i=comp($quartz,12000,$arm);
}

if ( ($all && $if) || $if) {
	print "\n>>> Results for an RX with $if Hz IF <<<\n";
	$i=comp($quartz,$if,$arm);
}

if ($all || $sdr) {
	printf("\n>>> Results for a Software Defined Radio <<<\n");
	printf(">>>  Johnson counter - Center frequency  <<<\n");
	$i=SDcomp($quartz,$arm,$sampleR);
}

if ($all || $drma) {
	printf("\n>>> Results for a DRM 455-12kHz Converter <<<\n");
	$i=DRMcomp($quartz,455000,32);
}

if ($all || $drmd) {
	printf("\n>>> Results for a DRM 9011-12kHz Converter <<<\n");
	$i=DRMcomp($quartz,9011000,32);
}

if ($all || $drmc) {
	printf("\n>>> Results for a DRM 10700-12kHz Converter <<<\n");
	$i=DRMcomp($quartz,10700000,32);
}

if ( ($all && $haveDRM) || ($drmdc && $haveDRM) ) {
	print "\n>>> Results for a direct-conversion DRM RX <<<\n";
	foreach $line (@qrg) {
        	$i=DRMcomp($quartz,$line*1000,$arm);
	}
} # end IF haveDRM

if ($all || $icsa) {
	printf("\n>>> Results for a IDT ICS512 chip (CW TX or DC RX) <<<\n");
	$i=ICS512comp($quartz);
}

if ($all || $icsb) {
	printf("\n>>> Results for a IDT ICS672 chip (CW TX or SDR RX) <<<\n");
	$i=ICS672comp($quartz);
}


if ($all || $mcw) {
	printf("\n>>> Results for an MCW optical system <<<\n");
	$i=MCWcomp($quartz);
}
printf("\n......................................................\n");
1;


# END MAIN
}




sub MCWcomp() {

  my $quartz = shift;

  my $i;
  my $res;

  # check harmonics
  for ($i=1;$i<=16;$i++) {
    $res=($quartz/(2**$i));
    if ($res < 4800) {
	    print "Divided by ". (2**$i) . " (". $i .") gives ". sprintf("%.0f",$res) ." Hz of modulated carrier\n";
    }
  }
}


sub SDcomp() {

  my $quartz = shift;
#  my $iff = shift;
  my $arm = shift;
  my $samplerate = shift;

  my $i;
  my $res;

  # check harmonics
  for ($i=1;$i<=$arm;$i++) {
    $res=inband($i*$quartz/4);
    if ($res) {print "It works on $res Hz, at $i harmonic (".($quartz*$i/1000000)." MHz), covering ".($res-($samplerate/2))."-".($res+($samplerate/2))." Hz\n";}
  }
}


sub ICS512comp() {
# ICS512 has integrated x2, x2.5, x3, x3.333, x4, x5, x5.333, x6 or x8 selections
# Input crystal frequency of 5 - 27 MHz
# Input clock frequency of 2 - 50 MHz
# Output clock frequencies up to 200 MHz

  my $quartz = shift;

  # if the input frequency is out of range
  if ( ($quartz < 2000000) || ($quartz > 50000000) ) { return 0; }

  my $res;

  # check multiplier factors
    $res=inband($quartz*2);
    if (($res)&&($res<200000000)) {print "It works on $res Hz, at x2 factor\n"};
    $res=inband($quartz*2.5);
    if (($res)&&($res<200000000)) {print "It works on $res Hz, at x2.5 factor\n"};
    $res=inband($quartz*3);
    if (($res)&&($res<200000000)) {print "It works on $res Hz, at x3 factor\n"};
    $res=inband($quartz*3.333);
    if (($res)&&($res<200000000)) {print "It works on $res Hz, at x3.333 factor\n"};
    $res=inband($quartz*4);
    if (($res)&&($res<200000000)) {print "It works on $res Hz, at x4 factor\n"};
    $res=inband($quartz*5);
    if (($res)&&($res<200000000)) {print "It works on $res Hz, at x5 factor\n"};
    $res=inband($quartz*5.3336);
    if (($res)&&($res<200000000)) {print "It works on $res Hz, at x5.333 factor\n"};
    $res=inband($quartz*6);
    if (($res)&&($res<200000000)) {print "It works on $res Hz, at x6 factor\n"};
    $res=inband($quartz*8);
    if (($res)&&($res<200000000)) {print "It works on $res Hz, at x8 factor\n"};
}


sub ICS672comp() {
# ICS672 has integrated x0.5, x1, x2, x3, x4, x5, or x6 selections
# Input clock range from 5 MHz to 150 MHz
# Outputs from up to 84 MHz (ICS672-01) and up to 135 MHz

  my $quartz = shift;

  # if the input frequency is out of range
  if ( ($quartz < 5000000) || ($quartz > 150000000) ) { return 0; }

  my $res;

  # check multiplier factors
    $res=inband($quartz*0.5);
    if (($res)&&($res<150000000)) {print "It works on $res Hz, at x0.5 factor\n"};
    $res=inband($quartz*1);
    if (($res)&&($res<150000000)) {print "It works on $res Hz, at x1 factor\n"};
    $res=inband($quartz*2);
    if (($res)&&($res<150000000)) {print "It works on $res Hz, at x2 factor\n"};
    $res=inband($quartz*3);
    if (($res)&&($res<150000000)) {print "It works on $res Hz, at x3 factor\n"};
    $res=inband($quartz*4);
    if (($res)&&($res<150000000)) {print "It works on $res Hz, at x4 factor\n"};
    $res=inband($quartz*5);
    if (($res)&&($res<150000000)) {print "It works on $res Hz, at x5 factor\n"};
    $res=inband($quartz*6);
    if (($res)&&($res<150000000)) {print "It works on $res Hz, at x6 factor\n"};

} #end sub


sub DRMcomp() {

  my $quartz = shift;
  my $if = shift;
  my $arm = shift;

  my $i;
  my $res;

  # check harmonics
  for ($i=1;$i<=$arm;$i++) {
    $res=$quartz*$i-$if;
    if (($res<20000)&&($res>-20000)) {print "At $i harmonic brings ".($if/1000)." kHz to ".abs($res)." Hz\n";}
    $res=$quartz/$i-$if;
    if (($res<20000)&&($res>-20000)) {print "Divided by $i brings ".($if/1000)." kHz to ".abs($res)." Hz\n";}
  }
}


sub comp() {

  my $quartz = shift;
  my $iff = shift;
  my $arm = shift;

  my $i;
  my $res;

  # check harmonics
  for ($i=1;$i<=$arm;$i++) {
    $res=inband($i*$quartz+$iff);
    if ($res) {print "It works on $res Hz, at $i harmonic\n";}
    if ($iff) {
      $res=inband($i*$quartz-$iff);
      if ($res) {print "It works on $res Hz, at $i harmonic\n";}
    }
  }

  # check for sub-harmonics
  for ($i=2;$i<=$arm;$i=$i+1) {
    $res=inband($quartz/$i+$iff);
    if ($res) {printf("It works on %d Hz, if divided by %d\n",$res,$i);}
    if ($iff) {
    	$res=inband($quartz/$i-$iff);
        if ($res) {printf("It works on %d Hz, if divided by %d\n",$res,$i);}
   }
  }

  return 0;

}



sub inband() {

  my $qrg = shift;

#  /* Here you can add or customize frequency bands for which *
#   * the software will check your XTALs                      */

  if (($qrg>=135000)&&($qrg<=139000)) {
    return $qrg;
  }
  if (($qrg>=1820000)&($qrg<=1850000)) {
    return $qrg;
  }
  if (($qrg>=3500000)&($qrg<=3800000)) {
    return $qrg;
  }
  if (($qrg>=7000000)&($qrg<=7100000)) {
    return $qrg;
  }
  if (($qrg>=10100000)&($qrg<=10150000)) {
    return $qrg;
  }
  if (($qrg>=14000000)&($qrg<=14350000)) {
    return $qrg;
  }
  if (($qrg>=18068000)&($qrg<=18168000)) {
    return $qrg;
  }
  if (($qrg>=21000000)&($qrg<=21450000)) {
    return $qrg;
  }
  if (($qrg>=24890000)&($qrg<=24990000)) {
    return $qrg;
  }
  if (($qrg>=28000000)&($qrg<=29500000)) {
    return $qrg;
  }
  if (($qrg>=50000000)&($qrg<=51000000)) {
    return $qrg;
  }
  if (($qrg>=70087500)&($qrg<=70112500)) {
    return $qrg;
  }
  if (($qrg>=70187500)&($qrg<=70212500)) {
    return $qrg;
  }
  if (($qrg>=70287500)&($qrg<=70312500)) {
    return $qrg;
  }
  if (($qrg>=144000000)&($qrg<=146000000)) {
    return $qrg;
  }
  if (($qrg>=430000000)&($qrg<=438000000)) {
    return $qrg;
  }

  return 0;
}


# Print output heading
sub printbanner() {
                printf("\n         xtalfind ik1zyw's XTAL utility");
                printf("\n                 version: $version\n\n");
}

